/*----------------------------------------------------------------------
  File    : sodim.h
  Contents: subset size occurrence distribution based item set mining
  Author  : Christian Borgelt
  History : 2014.08.26 file created
            2014.08.28 functions sod_data() and sod_report() added
----------------------------------------------------------------------*/
#ifndef __SODIM__
#define __SODIM__
#include "report.h"

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- similarity measures --- */
#define SOD_NONE             0  /* always 0 */
#define SOD_RUSSEL_RAO       1  /* S_R = s/n */
#define SOD_KULCZYNSKI       2  /* S_K = s/q */
#define SOD_JACCARD          3  /* S_J = s/r */
#define SOD_TANIMOTO         3
#define SOD_DICE             4  /* S_D = 2s/(r+s) */
#define SOD_SORENSEN         4
#define SOD_CZEKANOWKSI      4
#define SOD_SOKAL_SNEATH_1   5  /* S_S = s/(r+q) */
#define SOD_SOKAL_MICHENER   6  /* S_M = (s+z)/n */
#define SOD_HAMMING          6
#define SOD_FAITH            7  /* S_F = (s+z/2)/n */
#define SOD_ROGERS_TANIMOTO  8  /* S_T = (s+z)/(n+q) */
#define SOD_SOKAL_SNEATH_2   9  /* S_N = 2(s+z)/(n+s+z) */
#define SOD_GOWER_LEGENDRE   9
#define SOD_SOKAL_SNEATH_3  10  /* S_O = (s+z)/q */
#define SOD_BARONI_BUSER    11  /* S_B = (x+s)/(x+r) */
#define SOD_GENERIC         12  /* S   = (c_0s +c_1z +c_2n +c_3x)
                                       / (c_4s +c_5z +c_6n +c_7x) */

/* n = total number of transactions                          */
/* s = number of transactions with all items in the item set */
/* r = number of transactions with at least one item in set  */
/* q = number of transactions with some (but not all) items  */
/* z = number of transactions with no items in the set       */
/* x = \sqrt{sz} (geometric mean of s and z)                 */

/* --- evaluation measures --- */
#define SOD_LDRATIO 1           /* binary log. of support quotient */
#define SOD_SIZESIM 2           /* item set size times similarity */

/* --- operation modes --- */
#define SOD_PERFECT 0x0010      /* prune with perfect extensions */
#define SOD_TIDS    0x0020      /* report transaction identifiers */
#define SOD_DEFAULT SOD_PERFECT
#ifdef NDEBUG
#define SOD_NOCLEAN 0x8000      /* do not clean up memory */
#else                           /* in function jim() */
#define SOD_NOCLEAN 0           /* in debug version */
#endif                          /* always clean up memory */
#define SOD_VERBOSE INT_MIN     /* verbose message output */

/* --- algorithm variants --- */
#define SOD_BASIC   0           /* basic algorithm (dummy) */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern int sod_data   (TABAG *tabag, int target, SUPP smin, ITEM zmin,
                       int eval, int algo, int mode, int sort);
extern int sod_report (ISREPORT *report, int target,
                       int eval, double thresh, int algo, int mode);
extern int sodim      (TABAG *tabag, SUPP smin, double *xsps,int xspcnt,
                       int simcov, double *simps, double sim,
                       double sfrac, int eval, double thresh,
                       ITEM miscnt, int algo, int mode,
                       ISREPORT *report);
#endif
